import pytest
from quart import Quart

from quart_cors import route_cors


@pytest.fixture(name='app', scope='function')
def _app() -> Quart:
    app = Quart(__name__)

    @app.route('/')
    @route_cors()
    async def index() -> str:
        return 'Hello'

    return app


# These tests are based on https://www.w3.org/TR/cors section 6.1, and
# follow the logic given.

@pytest.mark.asyncio
async def test_no_origin(app: Quart) -> None:
    test_client = app.test_client()
    response = await test_client.get('/')
    assert 'Access-Control-Allow-Origin' not in response.headers


@pytest.mark.asyncio
@pytest.mark.parametrize('origin', ['http://notquart.com', 'http://Quart.com'])
async def test_origin_doesnt_match(app: Quart, origin: str) -> None:
    test_client = app.test_client()
    app.config['QUART_CORS_ALLOW_ORIGIN'] = ['http://quart.com']
    response = await test_client.get('/', headers={'Origin': origin})
    assert 'Access-Control-Allow-Origin' not in response.headers


@pytest.mark.asyncio
async def test_credentials_and_wildcard(app: Quart) -> None:
    test_client = app.test_client()
    app.config['QUART_CORS_ALLOW_CREDENTIALS'] = True
    response = await test_client.get('/', headers={'Origin': 'http://quart.com'})
    assert response.status_code == 500


@pytest.mark.asyncio
async def test_credentials(app: Quart) -> None:
    test_client = app.test_client()
    app.config['QUART_CORS_ALLOW_ORIGIN'] = ['http://quart.com']
    app.config['QUART_CORS_ALLOW_CREDENTIALS'] = True
    response = await test_client.get('/', headers={'Origin': 'http://quart.com'})
    assert response.access_control.allow_origin == {'http://quart.com'}
    assert response.vary == {'Origin'}
    assert response.access_control.allow_credentials


@pytest.mark.asyncio
async def test_expose_headers(app: Quart) -> None:
    test_client = app.test_client()
    app.config['QUART_CORS_EXPOSE_HEADERS'] = ['X-Special', 'X-Other']
    response = await test_client.get('/', headers={'Origin': 'http://quart.com'})
    assert response.access_control.allow_origin == {'*'}
    assert response.access_control.expose_headers == {'X-Special', 'X-Other'}



import pytest
from quart import Quart

from quart_cors import route_cors


@pytest.fixture(name='app', scope='function')
def _app() -> Quart:
    app = Quart(__name__)

    @app.route('/')
    @route_cors()
    async def index() -> str:
        return 'Hello'

    return app


# These tests are based on https://www.w3.org/TR/cors section 6.2, and
# follow the logic given.

@pytest.mark.asyncio
async def test_no_origin(app: Quart) -> None:
    test_client = app.test_client()
    response = await test_client.options('/')
    assert 'Access-Control-Allow-Origin' not in response.headers


@pytest.mark.asyncio
@pytest.mark.parametrize('origin', ['http://notquart.com', 'http://Quart.com'])
async def test_origin_doesnt_match(app: Quart, origin: str) -> None:
    test_client = app.test_client()
    app.config['QUART_CORS_ALLOW_ORIGIN'] = ['http://quart.com']
    response = await test_client.options('/', headers={'Origin': origin})
    assert 'Access-Control-Allow-Origin' not in response.headers


@pytest.mark.asyncio
async def test_request_method_doesnt_match(app: Quart) -> None:
    test_client = app.test_client()
    app.config['QUART_CORS_ALLOW_METHODS'] = ['GET', 'POST']
    response = await test_client.options(
        '/', headers={'Origin': 'http://quart.com', 'Access-Control-Request-Method': 'DELETE'},
    )
    assert response.access_control.allow_origin == {'*'}
    assert 'Access-Control-Allow-Headers' not in response.headers


@pytest.mark.asyncio
async def test_request_method_match(app: Quart) -> None:
    test_client = app.test_client()
    app.config['QUART_CORS_ALLOW_METHODS'] = ['GET', 'POST']
    response = await test_client.options(
        '/', headers={'Origin': 'http://quart.com', 'Access-Control-Request-Method': 'POST'},
    )
    assert response.access_control.allow_origin == {'*'}
    assert response.access_control.allow_methods == {'GET', 'POST'}


@pytest.mark.asyncio
async def test_request_headers(app: Quart) -> None:
    test_client = app.test_client()
    app.config['QUART_CORS_ALLOW_HEADERS'] = ['X-Match', 'X-Other']
    response = await test_client.options(
        '/',
        headers={
            'Origin': 'http://quart.com',
            'Access-Control-Request-Method': 'POST',
            'Access-Control-Request-Headers': 'X-Match, X-No-Match',
        },
    )
    assert response.access_control.allow_origin == {'*'}
    assert response.access_control.allow_headers == {'X-Match'}



from datetime import timedelta

import pytest
from quart import Blueprint, Quart

from quart_cors import cors, route_cors


@pytest.fixture(name='route_cors_app')
def _route_cors_app() -> Quart:
    app = Quart(__name__)

    @app.route('/')
    @route_cors(max_age=timedelta(seconds=5))
    async def index() -> str:
        return 'Hello'

    return app


@pytest.mark.asyncio
async def test_simple_cross_origin_request(route_cors_app: Quart) -> None:
    test_client = route_cors_app.test_client()
    response = await test_client.get('/', headers={'Origin': 'https://quart.com'})
    assert response.access_control.allow_origin == {'*'}


@pytest.mark.asyncio
async def test_preflight_request(route_cors_app: Quart) -> None:
    test_client = route_cors_app.test_client()
    response = await test_client.options(
        '/',
        headers={
            'Origin': 'https://quart.com',
            'Access-Control-Request-Method': 'DELETE',
        },
    )
    assert response.access_control.allow_origin == {'*'}
    assert response.access_control.allow_methods == {
        'GET', 'HEAD', 'POST', 'OPTIONS', 'PUT', 'PATCH', 'DELETE',
    }
    assert response.access_control.max_age == 5


@pytest.mark.asyncio
async def test_app_cors() -> None:
    app = Quart(__name__)

    @app.route('/')
    async def index() -> str:
        return 'Hello'

    app = cors(app)

    test_client = app.test_client()
    response = await test_client.get('/', headers={'Origin': 'https://quart.com'})
    assert response.access_control.allow_origin == {'*'}


@pytest.mark.asyncio
async def test_blueprint_cors() -> None:
    app = Quart(__name__)

    blueprint = Blueprint('name', __name__)

    blueprint = cors(blueprint)

    @blueprint.route('/')
    async def index() -> str:
        return 'Hello'

    app.register_blueprint(blueprint)

    test_client = app.test_client()
    response = await test_client.get('/', headers={'Origin': 'https://quart.com'})
    assert response.access_control.allow_origin == {'*'}



import pytest
from quart import Blueprint, Quart

from quart_cors import cors, route_cors


@pytest.fixture(name='app')
def _app() -> Quart:
    app = Quart(__name__)
    app = cors(app, allow_origin='http://app.com')
    app.config['QUART_CORS_ALLOW_METHODS'] = ['GET', 'POST']

    blueprint = Blueprint('blue', __name__)
    blueprint = cors(blueprint, allow_origin=['http://blueprint.com'])

    @app.route('/app')
    async def app_route() -> str:
        return 'App'

    @app.route('/route')
    @route_cors(allow_origin=['http://route.com'])
    async def route() -> str:
        return 'Route'

    @blueprint.route('/blueprint')
    async def blueprint_() -> str:
        return 'Blueprint'

    @blueprint.route('/blueprint_route')
    @route_cors(allow_origin=['http://blueprint.route.com'])
    async def blueprint_route() -> str:
        return 'Blueprint Route'

    app.register_blueprint(blueprint)

    return app


@pytest.mark.asyncio
@pytest.mark.parametrize(
    'origin, path',
    [
        ('http://app.com', '/app'),
        ('http://route.com', '/route'),
        ('http://blueprint.com', '/blueprint'),
        ('http://blueprint.route.com', '/blueprint_route'),
    ],
)
async def test_match(app: Quart, origin: str, path: str) -> None:
    test_client = app.test_client()
    response = await test_client.options(
        path, headers={'Origin': origin, 'Access-Control-Request-Method': 'POST'},
    )
    assert response.access_control.allow_origin == {origin}
    assert response.access_control.allow_methods == {'GET', 'POST'}


@pytest.mark.asyncio
@pytest.mark.parametrize(
    'origin, path',
    [
        ('http://app.com', '/route'),
        ('http://route.com', '/app'),
        ('http://blueprint.com', '/blueprint_route'),
        ('http://blueprint.route.com', '/blueprint'),
    ],
)
async def test_no_match(app: Quart, origin: str, path: str) -> None:
    test_client = app.test_client()
    response = await test_client.options(
        path, headers={'Origin': origin, 'Access-Control-Request-Method': 'POST'},
    )
    assert 'Access-Control-Allow-Origin' not in response.headers
